@@ -53,15 +53,14 @@ def do_check_migration(config, cmd):
     validate_head_file(config)
 
 
-def do_upgrade_downgrade(config, cmd):
+def do_upgrade(config, cmd):
     if not CONF.command.revision and not CONF.command.delta:
         raise SystemExit(_('You must provide a revision or relative delta'))
 
     revision = CONF.command.revision
 
     if CONF.command.delta:
-        sign = '+' if CONF.command.name == 'upgrade' else '-'
-        revision = sign + str(CONF.command.delta)
+        revision = '+%s' % str(CONF.command.delta)
     else:
         revision = CONF.command.revision
 
@@ -121,12 +120,11 @@ def add_command_parsers(subparsers):
     parser = subparsers.add_parser('check_migration')
     parser.set_defaults(func=do_check_migration)
 
-    for name in ['upgrade', 'downgrade']:
-        parser = subparsers.add_parser(name)
-        parser.add_argument('--delta', type=int)
-        parser.add_argument('--sql', action='store_true')
-        parser.add_argument('revision', nargs='?')
-        parser.set_defaults(func=do_upgrade_downgrade)
+    parser = subparsers.add_parser('upgrade')
+    parser.add_argument('--delta', type=int)
+    parser.add_argument('--sql', action='store_true')
+    parser.add_argument('revision', nargs='?')
+    parser.set_defaults(func=do_upgrade)
 
     parser = subparsers.add_parser('stamp')
     parser.add_argument('--sql', action='store_true')
