@@ -48,7 +48,7 @@ class VNFInUse(exceptions.InUse):
 
 
 class InvalidInfraDriver(exceptions.InvalidInput):
-    message = _('VIM type %(vim_name)s is not supported as a infra driver ')
+    message = _('VIM type %(vim_name)s is not supported as an infra driver ')
 
 
 class InvalidServiceType(exceptions.InvalidInput):
@@ -67,8 +67,8 @@ class VNFScaleWaitFailed(exceptions.TackerException):
     message = _('%(reason)s')
 
 
-class VNFDeleteFailed(exceptions.TackerException):
-    message = _('deleting VNF %(vnf_id)s failed')
+class VNFDeleteWaitFailed(exceptions.TackerException):
+    message = _('%(reason)s')
 
 
 class VNFDNotFound(exceptions.NotFound):
